﻿using FTD2XX_NET;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace PCL6125_EB_Sample
{
    public class accessPCL
    {
        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// クラス／テーブル
        /// </summary>
        private clsFTDI hFTDI = new clsFTDI();

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// ステータス、サブステータスの読出し
        /// Read status, sub status
        ///   subS ： 0=status  1=sub status
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Read_STATUS(ref byte[] FtBuff, ref int FtIndex, int subS)
        {
            //CS=Low
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x0B;
            //
            FtBuff[FtIndex++] = 0x35; // SPI(Out on negative edge, in on negative edge)
            FtBuff[FtIndex++] = 0x04; // Length L 5byte
            FtBuff[FtIndex++] = 0x00; // Length H
            if(subS == 0)
            {
                FtBuff[FtIndex++] = 0x13; // command status read X,Y Axis
            }
            else
            {
                FtBuff[FtIndex++] = 0x33; // command sub status read X,Y Axis
            }
            FtBuff[FtIndex++] = 0x00; // X-axis dumy
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00; // Y-axis dumy
            FtBuff[FtIndex++] = 0x00;
            //CS=High
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x08;
            FtBuff[FtIndex++] = 0x0B;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// レジスタの読出し
        /// Read registers
        ///   comm ： レジスタリードコマンド
        ///           Register read command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Read_REG(ref byte[] FtBuff, ref int FtIndex, byte comm)
        {
            //CS=Low
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x0B;
            //
            FtBuff[FtIndex++] = 0x35; // SPI(Out on negative edge, in on negative edge)
            FtBuff[FtIndex++] = 0x09; // Length L 10byte
            FtBuff[FtIndex++] = 0x00; // Length H
            FtBuff[FtIndex++] = 0x03; // Command write
            FtBuff[FtIndex++] = comm;
            FtBuff[FtIndex++] = 0x00; // X-axis dumy
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00; // Y-axis dumy
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x00;
            //CS=High
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x08;
            FtBuff[FtIndex++] = 0x0B;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// レジスタの書込み
	/// Register write
        ///   RegD ： 書込みデータ
        ///           Write data
        ///   Jsc  ： 軸選択コマンド
        ///           Axis selection command
        ///   comm ： レジスタライトコマンド
        ///           Register write command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Write_REG(ref byte[] FtBuff, ref int FtIndex, uint RegD, byte Jsc, byte comm)
        {
            ////////////////////////////////////////////////////////////////////
            var bytes = new TBytes();

            bytes.Word = RegD;

            //CS=Low
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x0B;
            //
            FtBuff[FtIndex++] = 0x11; // SPI(Clock Data Bytes Out on -ve Clock Edge MSB First (no Read))
            FtBuff[FtIndex++] = 0x05; // Length L 6byte
            FtBuff[FtIndex++] = 0x00; // Length H
            FtBuff[FtIndex++] = Jsc;
            FtBuff[FtIndex++] = comm;
            FtBuff[FtIndex++] = bytes.Byte0; // Data 7-0
            FtBuff[FtIndex++] = bytes.Byte1; // Data 15-8
            FtBuff[FtIndex++] = bytes.Byte2; // Data 23-16
            FtBuff[FtIndex++] = bytes.Byte3; // Data 31-24
            //CS=High
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x08;
            FtBuff[FtIndex++] = 0x0B;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// コマンドの書込み
        /// Write command
        ///   Jsc  ： 軸選択コマンド
        ///           Axis selection command
        ///   comm ： コマンド
        ///           Command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Write_COM(ref byte[] FtBuff, ref int FtIndex, byte Jsc, byte comm)
        {
            ////////////////////////////////////////////////////////////////////
            //CS=Low
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x00;
            FtBuff[FtIndex++] = 0x0B;
            //
            FtBuff[FtIndex++] = 0x11; // SPI(Clock Data Bytes Out on -ve Clock Edge MSB First (no Read))
            FtBuff[FtIndex++] = 0x01; // Length L 2byte
            FtBuff[FtIndex++] = 0x00; // Length H
            FtBuff[FtIndex++] = Jsc;
            FtBuff[FtIndex++] = comm;
            //CS=High
            FtBuff[FtIndex++] = 0x80;
            FtBuff[FtIndex++] = 0x08;
            FtBuff[FtIndex++] = 0x0B;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// 積み込んだデータをUSB送信する
        /// Sending loaded data from USB
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS SendUsb(ref byte[] FtBuff, ref int FtIndex)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            if (FtIndex > 0)
            {
                ftStatus = hFTDI.LSI_Write(FtBuff, FtIndex);
                FtIndex = 0;
            }
            //
            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// USB側のデータを送信させる
        /// Transmit the USB side data
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS GetUsb(ref byte[] FtBuff)
        {
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            uint numBytesRead = 0;

            ////////////////////////////////////////////////////////////////////
            ftStatus = hFTDI.LSI_Read(ref FtBuff, ref numBytesRead);
            //
            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// 4byte 共用体
        /// 4byte Union
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        [StructLayout(LayoutKind.Explicit)]
        public struct TBytes
        {
            [FieldOffset(0)]
            public byte Byte0;

            [FieldOffset(1)]
            public byte Byte1;

            [FieldOffset(2)]
            public byte Byte2;

            [FieldOffset(3)]
            public byte Byte3;

            [FieldOffset(0)]
            public uint Word;
        }

        /// <summary>
        /// バッファからのデータの切り出し
        /// Cutting out data from buffer
        /// </summary>
        /// <param name="buff"></param>
        /// <param name="start"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public uint GetBuffData(byte[] buff, int start, int type)
        {
            var bytes = new TBytes();

            bytes.Byte0 = buff[start++];
            bytes.Byte1 = buff[start++];
            if(type == 0) // 2byte
            {
                bytes.Byte2 = 0;
                bytes.Byte3 = 0;
            }
            else //4byte
            {
                bytes.Byte2 = buff[start++];
                bytes.Byte3 = buff[start++];
            }
            return(bytes.Word);
        }
    }
}
